#include "hmbdc/Copyright.hpp"
#pragma once 


#include "hmbdc/app/Client.hpp"
#include "hmbdc/app/Message.hpp"
#include "hmbdc/comm/Topic.hpp"

#include "hmbdc/pattern/PoolT.hpp"
#include "hmbdc/pattern/LockFreeBufferT.hpp"
#include "hmbdc/pattern/MonoLockFreeBuffer.hpp"

#include "hmbdc/os/Thread.hpp"
#include "hmbdc/time/Timers.hpp"
#include "hmbdc/numeric/BitMath.hpp"


#include <vector>
#include <type_traits>
#include <thread>
#include <utility>
#include <iostream>

namespace hmbdc { namespace app { namespace detail {
template <typename... ContextProperties>
struct context_property_aggregator{
    using Buffer = hmbdc::pattern::LockFreeBufferT<DEFAULT_HMBDC_CAPACITY>;
    using Allocator = os::DefaultAllocator;
    enum {
        has_pool = 1,
        can_start_anytime = 0,
        create_ipc = 0,
        attach_ipc = 0,
    };
};

template <uint16_t c, typename... ContextProperties>
struct context_property_aggregator<context_property::broadcast<c>
    , ContextProperties...>
: context_property_aggregator<ContextProperties...> {
    using Buffer = hmbdc::pattern::LockFreeBufferT<c>;
    enum {
        has_pool = 1,
        can_start_anytime = 0
    };
};

template <typename... ContextProperties>
struct context_property_aggregator<context_property::partition
    , ContextProperties...> 
: context_property_aggregator<ContextProperties...> {
    using Buffer = hmbdc::pattern::MonoLockFreeBuffer;
    enum {
        has_pool = 0,
        can_start_anytime = 1
    };
};


template <typename... ContextProperties>
struct context_property_aggregator<context_property::ipc_creator
    , ContextProperties...> 
: context_property_aggregator<ContextProperties...> {
    using Allocator = os::ShmBasePtrAllocator;
    enum {
        attach_ipc = 0,
        create_ipc = 1,
    };
};

template <typename... ContextProperties>
struct context_property_aggregator<context_property::ipc_attacher
    , ContextProperties...> 
: context_property_aggregator<ContextProperties...> {
    using Allocator = os::ShmBasePtrAllocator;
    enum {
        create_ipc = 0,
        attach_ipc = 1,
    };
};

using namespace std;
using namespace hmbdc::time;

template <bool is_timer_manager>
struct tm_runner {
    template<typename C>
    void operator()(C&) {}
};

template <>
struct tm_runner<true> {
    void operator()(TimerManager& tm) {
        tm.checkTimers(SysTime::now());
    }
};

template <typename LFB, typename Client>
bool runOnceImpl(uint16_t threadSerialNumber, bool& __restrict__ stopped, LFB& __restrict__ lfb, Client& __restrict__ c) {
    typename LFB::iterator begin, end;
    try {
        tm_runner<std::is_base_of<TimerManager, Client>::value> tr;
        tr(c);

        const bool clientParticipateInMessaging = 
            std::remove_reference<Client>::type::REGISTERED_MESSAGE_SIZE;
        if (clientParticipateInMessaging) {
            uint64_t count = lfb.peek(threadSerialNumber, begin, end, c.maxBatchMessageCount());
            c.Client::handleRangeImpl(begin, end, threadSerialNumber);
            c.Client::invokedCb(threadSerialNumber);
            lfb.wasteAfterPeek(threadSerialNumber, count);
        } else {
            c.Client::invokedCb(0xffffu - threadSerialNumber);
        }
    } catch (std::exception const& e) {
        if (!stopped) {
            c.stopped(e);
            return !c.dropped();
        }
    } catch (int code) {
        if (!stopped) {
            c.stopped(hmbdc::ExitCode(code));
            return !c.dropped();
        }
    } catch (...) {
        if (!stopped) {
            c.stopped(hmbdc::UnknownException());
            return !c.dropped();
        }
    }
    return true;
} 

template <typename Client>
bool runOnceImpl(uint16_t threadSerialNumber, bool& __restrict__ stopped, hmbdc::pattern::MonoLockFreeBuffer& __restrict__ lfb, Client& __restrict__ c) {
    hmbdc::pattern::MonoLockFreeBuffer::iterator begin, end;
    try {
        tm_runner<std::is_base_of<TimerManager, Client>::value> tr;
        tr(c);

        const bool clientParticipateInMessaging = 
            std::remove_reference<Client>::type::REGISTERED_MESSAGE_SIZE;
        if (clientParticipateInMessaging) {
            uint64_t count = lfb.peek(begin, end, c.maxBatchMessageCount());
            auto b = begin;
            c.Client::handleRangeImpl(b, end, threadSerialNumber);
            c.Client::invokedCb(threadSerialNumber);
            lfb.wasteAfterPeek(begin, count);
        } else {
            c.Client::invokedCb(0xffffu - threadSerialNumber);
        }
    } catch (std::exception const& e) {
        if (!stopped) {
            c.stopped(e);
            return !c.dropped();
        }
    } catch (int code) {
        if (!stopped) {
            c.stopped(hmbdc::ExitCode(code));
            return !c.dropped();
        }
    } catch (...) {
        if (!stopped) {
            c.stopped(hmbdc::UnknownException());
            return !c.dropped();
        }
    }
    return true;
}

void unblock(MonoLockFreeBuffer& buffer, uint16_t){
    buffer.reset();
}

template <typename Buffer>
void unblock(Buffer& lfb, uint16_t threadSerialNumber) {
    lfb.markDead(threadSerialNumber);
}

}}}
