#include "hmbdc/Copyright.hpp"
#pragma once 

#include "hmbdc/comm/Topic.hpp"
#include "hmbdc/time/Time.hpp"
#include "hmbdc/app/Message.hpp"

#include <ostream>

namespace hmbdc { namespace app { namespace mcast {

using namespace hmbdc; 
using namespace hmbdc::comm;

struct TransportMessageHeader {
    uint8_t topicLen;

    uint16_t& messagePayloadLen() {
        return *reinterpret_cast<uint16_t*>(payload());
    }

    uint16_t const& messagePayloadLen()  const {
        return *reinterpret_cast<uint16_t const*>(payload());
    }

    std::pair<char const*, char const*> topic() const {
        char const* b = reinterpret_cast<const char*>(this) 
            + sizeof(TransportMessageHeader); 
        return std::make_pair(b, b + topicLen);
    }

    void const* payload() const {
        return reinterpret_cast<const char*>(this) 
            + sizeof(TransportMessageHeader) + topicLen; 
    }

    void * payload() {
        return reinterpret_cast<char*>(this) 
            + sizeof(TransportMessageHeader) + topicLen; 
    }

    uint16_t typeTag() const {
        auto h = static_cast<app::MessageHead const*>(payload());
        return h->typeTag;
    }

    template <typename Message>
    Message& wrapped() {
        auto wrap = static_cast<app::MessageWrap<Message>*>(payload());
        return wrap->payload;
    }

    template <typename Message>
    Message const& wrapped() const {
        auto wrap = static_cast<app::MessageWrap<Message> const *>(payload());
        return wrap->payload;
    }

    size_t wireSize() const {
        return sizeof(TransportMessageHeader) + topicLen 
            + messagePayloadLen();
    }    

    size_t wireSizeContainsTopic() const {
        return sizeof(TransportMessageHeader) + topicLen;
    }
} __attribute__((packed));

struct Subscribe
: hasTag<101> {
    Subscribe(Topic const&topic)
    : topic(topic){}
    Topic topic;
};

struct Unsubscribe 
: hasTag<102> {
    Unsubscribe(Topic const&topic)
    : topic(topic){}
    Topic topic;
};

}}}
