#include "hmbdc/Copyright.hpp"
#pragma once

#include "hmbdc/app/Config.hpp"
#include "hmbdc/text/StringTrieSet.hpp"
#include "hmbdc/comm/Topic.hpp"
#include "hmbdc/comm/inet/Misc.hpp"
#include "hmbdc/time/Timers.hpp"
#include "hmbdc/pattern/LockFreeBufferT.hpp"
#include "hmbdc/Compile.hpp"

#include <boost/regex.hpp>
#include <boost/asio.hpp>

#include <memory>

extern thread_local std::shared_ptr<boost::asio::io_service> gtIos;

namespace hmbdc { namespace app { namespace mcast {

using namespace hmbdc;
using namespace hmbdc::time;
using namespace hmbdc::comm;
using namespace boost::asio;
struct Transport {
    using ptr = std::shared_ptr<Transport>;

    Transport(Config const& cfg)
    : config_(cfg)
    , pIos_(nullptr) {
        cfg (hmbdcName_, "hmbdcName")
            (schedPolicy_, "schedPolicy")
            (schedPriority_, "schedPriority")
        ;
    }

    void initInThread() {
        if (!::gtIos) {
            ::gtIos.reset(new io_service(1u));
        }
        pIos_ = ::gtIos;
    }

    bool operator == (Transport const& other ) const {
        return &config_ == &other.config_;
    }

    bool operator < (Transport const& other ) const {
        return &config_ < &other.config_;
    }

    virtual ~Transport(){
    }

protected:
    string hmbdcName_;
    string schedPolicy_;
    int schedPriority_;
    using Buffer = hmbdc::pattern::LockFreeBufferT<1>;
    Config const config_;
    std::shared_ptr<io_service> pIos_;
};
}}}
