#include "hmbdc/Copyright.hpp"
#pragma once 

#include "hmbdc/comm/Topic.hpp"
#include "hmbdc/time/Time.hpp"
#include "hmbdc/app/Message.hpp"

#include <ostream>
#include <string>

#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

namespace hmbdc { namespace app { namespace tcpcast {

using namespace hmbdc; 
using namespace hmbdc::comm;


struct hasMemoryAttachment {
    enum {
        flag = 1
    };
    size_t len;
    void* attachment;
    void free() {
        ::free(attachment);
        attachment = nullptr;
    }
} __attribute__((aligned (8)));

struct hasFileAttachment {
    enum {
        flag = 2
    };
    char file[16];
    size_t len;
    void* attachment;
    size_t map(char const* fileName) {
        using namespace std;
        int fd;
        struct stat sb;
     
        fd = open(fileName, O_RDONLY);
        fstat(fd, &sb);
        len = (uint64_t)sb.st_size;
        attachment = mmap(NULL, sb.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
        if (attachment == MAP_FAILED) return 0;
        close(fd);

        string line(fileName);
        auto pos = line.find_last_of('/');
        if (pos == string::npos) pos = 0;
        strncpy(file, line.c_str() + pos + 1, sizeof(file));
        return len;
    }

    void unmap() {
        munmap(attachment, len);
        attachment = nullptr;
    }
} __attribute__((aligned (8)));

struct TransportMessageHeader {
    uint8_t topicLen;
    uint8_t flag; //1 - hasAttachment
    uint16_t messagePayloadLen;

    std::pair<char const*, char const*> topic() const {
        char const* b = reinterpret_cast<const char*>(this) 
            + sizeof(TransportMessageHeader); 
        return std::make_pair(b, b + topicLen);
    }

    void const* payload() const {
        return reinterpret_cast<const char*>(this) 
            + sizeof(TransportMessageHeader) + topicLen; 
    }

    void * payload() {
        return reinterpret_cast<char*>(this) 
            + sizeof(TransportMessageHeader) + topicLen; 
    }

    uint16_t typeTag() const {
        auto h = static_cast<app::MessageHead const*>(payload());
        return h->typeTag;
    }

    template <typename Message>
    Message& wrapped() {
        auto wrap = static_cast<app::MessageWrap<Message>*>(payload());
        return wrap->payload;
    }

    template <typename Message>
    Message const& wrapped() const {
        auto wrap = static_cast<app::MessageWrap<Message> const *>(payload());
        return wrap->payload;
    }

    size_t wireSize() const {
        return sizeof(TransportMessageHeader) + topicLen 
            + messagePayloadLen;
    }    

    size_t wireSizeContainsTopic() const {
        return sizeof(TransportMessageHeader) + topicLen;
    }
} __attribute__((packed));

struct Subscribe
: hasTag<201> {
    Subscribe(Topic const&topic)
    : topic(topic){}
    Topic topic;
};

struct Unsubscribe 
: hasTag<202> {
    Unsubscribe(Topic const&topic)
    : topic(topic){}
    Topic topic;
};

struct TopicSource 
: hasTag<203> {
    TopicSource(std::string const&topicRegexIn
        , std::string const& ipIn
        , uint16_t portIn
        , bool loopbackIn)
    : port(portIn)
    , pid(getpid())
    , loopback(loopbackIn)
    , timestamp(time::SysTime::now()) {
        strncpy(ip, ipIn.c_str(), sizeof(ip));
        strncpy(topicRegex, topicRegexIn.c_str(), sizeof(topicRegex));
        topicRegex[sizeof(topicRegex) - 1] = 0;
    }
    char ip[16];
    char topicRegex[128]; //null terminated string
    uint16_t port;
    pid_t pid;
    bool loopback;
    time::SysTime timestamp; //when the source started
};

struct SessionStarted 
: hasTag<204> {
    char ip[16];
    friend 
    std::ostream& operator << (std::ostream& os, SessionStarted const & m) {
        return os << "Session to topic source started " << m.ip;
    }
};

struct SessionDropped 
: hasTag<205> {
    char ip[16];
    friend 
    std::ostream& operator << (std::ostream& os, SessionDropped const & m) {
        return os << "Session to topic source dropped " << m.ip;
    }
};

}}}
