#include <iostream>
#include "hmbdc/app/utils/ConsoleClient.hpp"
#include "hmbdc/app/LoggerT.hpp"
#include "hmbdc/app/Client.hpp"
#include "hmbdc/app/Context.hpp"
#include "hmbdc/app/Config.hpp"
#include "hmbdc/os/Signals.hpp"
#include "hmbdc/numeric/BitMath.hpp"

#include <boost/program_options.hpp>
#include <memory>
#include <unistd.h>
#include <signal.h>

using namespace std;
using namespace hmbdc;
using namespace hmbdc::numeric;
using namespace hmbdc::app;
using namespace boost::property_tree;


namespace hmbdc { namespace app { namespace utils {
using LoggerContext = HMBDC_LOG_CONTEXT;
using MyContext = hmbdc::app::Context<>; //run time sized
using MyLogger = LoggerT<LoggerContext>;

template <typename NetContext>
void run(ptree const& configIn) {   
    Config config(configIn);
    using namespace std;
    using namespace boost;

    SingletonGuardian<MyLogger> logGuard(cerr);

    auto maxMessageSize = config.get<uint16_t>("maxMessageSize");

    MyContext ctx(config.get<uint16_t>("inBufferSizePower2")
        , config.get_child("transports").size() * 2 + 1 //max transports count + a Client
        , maxMessageSize
    );

    SingletonGuardian<NetContext> g;

    auto coreBitMask = config.getHex<uint64_t>("coreBitMask");

    //make sure coreBitMask is not zero
    if (coreBitMask == 0) {
        coreBitMask = (1ul << std::thread::hardware_concurrency()) - 1ul;
    }

    //start ctx and add its clients
    ctx.start(setBitsCount(coreBitMask), coreBitMask);
    // ctx.start(0, 0, true);

    for (auto const& t : config.get_child("transports")) {
        Config tconf(config, t.second);
        auto poolThreadAffinity = 1ul;
        if (t.second.get("tx", false)) {
            auto sengine = NetContext::instance().createSendTransportEngine(
                tconf, maxMessageSize);
            // ctx.start(*sengine, poolThreadAffinity, true);
            ctx.addToPool(*sengine, poolThreadAffinity);
        }
        if (t.second.get("rx", false)) {
            auto rengine = NetContext::instance().createRecvTransportEngine(
                tconf, ctx.buffer());
            ctx.addToPool(*rengine, poolThreadAffinity);
        }
        poolThreadAffinity <<= 1ul;
        //wrap around
        if (poolThreadAffinity == (1ul << setBitsCount(coreBitMask))) {
            poolThreadAffinity = 1ul;
        }
    }
    
    ConsoleClient<NetContext> client(ctx
        , cin
        , cout
        , cerr
        , config.get("initCmd", ""));

    ctx.addToPool(client);
    os::HandleSignals::onTermIntDo(
        [&ctx]() {
        ctx.stop();
    });
    
    client.waitUntilFinish();
    sleep(config.get("secWaitAfterInputDone", 1));
    ctx.stop();
    ctx.join();
}


template <typename NetContext>
int startConsole(char const* helpStrIn, int argc, char** argv) {
    using namespace std;
    if (argc != 2) {
        cout << helpStrIn << endl;
        exit(1);
    }

    ptree config;
    string userCfg(argv[1]);
    std::ifstream cs(userCfg);
    read_json(cs, config);
    run<NetContext>(config);
}

}}}

