#include "hmbdc/Copyright.hpp"
#pragma once 

#include "hmbdc/Exception.hpp"

#include <iostream>
#include <memory>
#include <algorithm>
#include <stdexcept>

// #include <ext/stdio_filebuf.h>
#include <string.h>
#include <unistd.h>

namespace hmbdc { namespace os {
using namespace std;
/**
 * @brief execute a program as a child and capture its stdin stdout and/or stderr
 * @details IO are exposed in the fd form
 */
struct ExecutionIo {
	/**
	 * @brief execute a program as a child and capture its stdin stdout and/or stderr
	 * 
	 * @param argv ullptr terminated command line args array, 
	 * for example, {"/usr/bin/bc", "-q", nullptr}
	 * @param captureStderr optionally capture
	 */
	ExecutionIo(const char* const argv[]
		, bool captureStderr = false);

	~ExecutionIo();

	/**
	 * @brief child stdin fd, use write on it - not read
	 * @details blocking by default
	 */
	int exeStdinFd;

	/**
	 * @brief child stdout fd, use read on it - not write
	 * @details blocking by default
	 */
	int exeStdoutFd;

    /**
     * @brief if captured, child stderr fd, use read on it - not write
     * @details blocking by default
     */
	int exeStderrFd;

	/**
     * @brief process id of the started child process
     */
	pid_t exePid;

	
	// ostream exeCin;
	// istream exeCout;
	// istream exeCerr;

	ExecutionIo& operator << (char const* input) {
		auto s = write(exeStdinFd, input, strlen(input));
		if ((size_t)s != strlen(input)) {
			HMBDC_THROW(runtime_error, "write failed, bytes written=" << s);
		}
		return *this;
	}

	ExecutionIo& operator << (string const& input) {
		auto s = write(exeStdinFd, input.c_str(), input.size());
		if ((size_t)s != input.size()) {
			HMBDC_THROW(runtime_error, "write failed, bytes written=" << s);
		}
		return *this;
	}

	ExecutionIo& operator >> (string& output) {
		for (;;) {
			auto l = std::find(buf_, buf_ + bufLen_, '\n');
			//is no \n in 1024 char buffer, just treat the last char as \n
			if (l == buf_ + sizeof(buf_)) l--; 
			if (l != buf_ + bufLen_) {
			    output = string(buf_, l + 1);
		        bufLen_ -= output.size();
		        memmove(buf_, l + 1, bufLen_);
			    return *this;
			}
			int s = read(exeStdoutFd, buf_ + bufLen_, sizeof(buf_) - bufLen_);
			bufLen_ += s;
		}
	}

	ExecutionIo& readErr(string& output) {
		char buf[1024];
		int s = read(exeStderrFd, buf, sizeof(buf));
		output = string(buf, s);
		return *this;
	}

private:
	char buf_[1024];
	size_t bufLen_;

// private:
// 	unique_ptr<__gnu_cxx::stdio_filebuf<char>> exeCinBuf_;
// 	unique_ptr<__gnu_cxx::stdio_filebuf<char>> exeCoutBuf_;
// 	unique_ptr<__gnu_cxx::stdio_filebuf<char>> exeCerrBuf_;
};
}}
