#include "hmbdc/Copyright.hpp"
#pragma once 

#include "hmbdc/Exception.hpp"
#include <stdexcept>
#include <type_traits>
#include <typeinfo>

namespace hmbdc { namespace pattern {

template <typename Singleton>
struct SingletonGuardian {
    template <typename...Args>
    SingletonGuardian(Args&&...);
    virtual ~SingletonGuardian();
};

template<typename Singleton>
struct GuardedSingleton {
    friend class SingletonGuardian<Singleton>;
    static Singleton& instance() {return *pInstance_s;}
    static bool initialized() {return pInstance_s;}

private:
    static Singleton* pInstance_s;
};

template <typename Singleton> Singleton*
    GuardedSingleton<Singleton>::pInstance_s = 0;

template <typename Singleton>
template <typename...Args>
SingletonGuardian<Singleton>::
SingletonGuardian(Args&&...args) {
    if (GuardedSingleton<Singleton>::pInstance_s) {
        HMBDC_THROW(std::runtime_error
            , "Cannot reinitialize typeid=" << typeid(Singleton).name());
    }
    GuardedSingleton<Singleton>::pInstance_s 
        = new Singleton(std::forward<Args>(args)...);
}

template <typename Singleton>
SingletonGuardian<Singleton>::
~SingletonGuardian() {
    delete GuardedSingleton<Singleton>::pInstance_s;
    GuardedSingleton<Singleton>::pInstance_s = 0;
}

}}


